@@ -5,6 +5,7 @@ from math import ceil
 from operator import itemgetter
 from random import randrange
 
+import graphviz
 import lime
 import lime.lime_tabular
 import matplotlib.pyplot as plt
@@ -21,7 +22,6 @@ from sklearn.feature_extraction.text import (
     TfidfVectorizer,
 )
 from sklearn.feature_selection import VarianceThreshold
-from sklearn.linear_model import ElasticNet, LogisticRegression, SGDClassifier
 from sklearn.metrics import (
     accuracy_score,
     f1_score,
@@ -33,10 +33,9 @@ from sklearn.metrics import (
     roc_auc_score,
 )
 from sklearn.model_selection import train_test_split
-from sklearn.pipeline import Pipeline
-from sklearn.tree import DecisionTreeClassifier
+from sklearn.tree import export_graphviz
 
-from datto.CleanText import CleanText
+from datto.CleanDataframe import CleanDataframe
 
 
 class ModelResults:
@@ -78,7 +77,7 @@ class ModelResults:
             | chosen_stopwords
         )
 
-        ct = CleanText()
+        ct = CleanDataframe()
         vectorizer = TfidfVectorizer(
             tokenizer=ct.lematize,
             ngram_range=(1, 3),
@@ -236,17 +235,19 @@ class ModelResults:
             model,
         )
 
-    def coefficients_graph(self, X_train, X_test, model, model_type, filename):
+    def coefficients_graph(
+        self, X_train, X_test, model, model_type, filename, path="../images/"
+    ):
         """
         Displays graph of feature importances.
 
         * Number of horizontal axis indicates magnitude of effect on
             target variable (e.g. affected by 0.25)
-        * Red/blue indicates feature value (increasing or decreasing feature 
+        * Red/blue indicates feature value (increasing or decreasing feature
             has _ effect)
-        * Blue & red mixed together indicate there isn't a clear 
+        * Blue & red mixed together indicate there isn't a clear
             effect on the target variable
-        * For classification - interpreting magnitude number / x axis - changes the 
+        * For classification - interpreting magnitude number / x axis - changes the
             predicted probability of y on average by _ percentage points (axis value * 100)
 
         Parameters
@@ -258,8 +259,8 @@ class ModelResults:
             'classification' or 'regression'
         filename: str
         """
-        if not os.path.exists("../images"):
-            os.makedirs("../images/")
+        if not os.path.exists(path):
+            os.makedirs(path)
 
         if model_type.lower() == "classification":
             f = lambda x: model.predict_proba(x)[:, 1]
@@ -275,7 +276,7 @@ class ModelResults:
         shap_values = explainer.shap_values(X_test_sample)
         shap.summary_plot(shap_values, X_test_sample)
         plt.tight_layout()
-        plt.savefig(f"../images/{filename}.png")
+        plt.savefig(f"{path}{filename}.png")
 
         return shap_values
 
@@ -467,7 +468,7 @@ class ModelResults:
         model_type: str
             'classification' or 'regression'
         multiclass: bool
-            
+
         Returns
         --------
         simplified_df: DataFrame
@@ -553,9 +554,28 @@ class ModelResults:
         num_samples,
         model_type,
         class_names=["False", "True"],
+        path="../images/",
     ):
-        if not os.path.exists("../images"):
-            os.makedirs("../images/")
+        """
+        Uses LIME to inspect an individual prediction and the features that influenced that prediction.
+
+        Parameters
+        --------
+        trained_model: sklearn model (already fitted)
+        X_train: pd.DataFrame
+        X_test: pd.DataFrame
+        id_col: str
+        num_samples: int
+        model_type: str
+        class_names: str
+        path: str
+
+        Returns
+        --------
+        features: list
+        """
+        if not os.path.exists(path):
+            os.makedirs(path)
 
         def model_preds_adjusted(data):
             if model_type.lower() == "classification":
@@ -579,7 +599,7 @@ class ModelResults:
                 mode="regression",
             )
 
-        for i in range(num_samples):
+        for _ in range(num_samples):
             user_idx = randrange(X_test.shape[0])
             exp = explainer.explain_instance(
                 np.array(X_test.iloc[user_idx]),
@@ -629,9 +649,39 @@ class ModelResults:
                     print(" * This is false: " + feature[0])
             exp.as_pyplot_figure()
             plt.tight_layout()
-            plt.savefig(f"../images/lime_graph_user_{user_id}.png")
+            plt.savefig(f"{path}lime_graph_user_{user_id}.png")
 
             print("\n\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n")
 
         return features_list
 
+    def get_tree_diagram(self, model, X_train, path="../images/"):
+        """
+        Save a diagram of a trained DecisionTree model
+
+        Parameters
+        --------
+        model: sklearn model (trained)
+        X_train: pd.DataFrame
+        path: str
+        """
+        # Exporting text form of decision tree
+        dot_data = export_graphviz(
+            model,
+            out_file=f"{path}decision-tree.dot",
+            feature_names=X_train.columns,
+            filled=True,
+            rounded=True,
+            special_characters=True,
+        )
+        graph = graphviz.Source(dot_data)
+
+        # Converting text to a visual png file
+        os.system(f"dot -Tpng {path}decision-tree.dot -o {path}decision-tree.png")
+
+        # If the file didn't write, try reinstalling graphviz
+        if not os.path.exists(f"{path}decision-tree.png"):
+            os.system("brew install graphviz")
+            os.system(f"dot -Tpng {path}decision-tree.dot -o {path}decision-tree.png")
+
+        return graph
